to setup
  ca
  set-default-shape turtles "arrow"
  cct 500
    [ ;; turtle headings must be multiples of 60
      set heading 60 * random 6
      ;; shift even columns down
      if pxcor mod 2 = 0
        [ set ycor ycor - 0.5 ] ]
end

to go
  ;; turtles do a random walk (usually going straight,
  ;; occasionally turning)
  ask turtles
    [ hex-fd
      rt random-one-of [-60 0 0 0 0 0 60] ]
end

to hex-fd  ;; turtle procedure
  ;; in the context of this example, the without-interruption
  ;; isn't really needed, but it might be in the context of
  ;; a full model
  without-interruption
    [ ;; This is made a little more complicated by the need to handle
      ;; wrapping properly.  If the turtles couldn't wrap around the
      ;; edges of the screen, we wouldn't need to use ROUND and IF here.
      ;; But since the total number of patch column is always odd, that
      ;; means that wrapping puts two even columns or two odd columns
      ;; "next" to each other, so we can't rely on even and odd columns
      ;; always alternating.
      setxy round (xcor + item (heading / 60) [0 1 1 0 -1 -1])
            round (ycor + item (heading / 60) [1 0.5 -0.5 -1 -0.5 0.5])
      if pxcor mod 2 = 0
        [ set ycor ycor - 0.5 ] ]
end


; (C) 2004 Uri Wilensky.  This code may be freely copied, distributed,
; altered, or otherwise used by anyone for any legal purpose.
; 
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
; OWNERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
@#$#@#$#@
GRAPHICS-WINDOW
240
10
775
566
10
10
25.0
1
10
1
1
1

CC-WINDOW
3
269
238
566
Command Center

BUTTON
82
70
163
103
NIL
setup
NIL
1
T
OBSERVER
T

BUTTON
82
138
163
171
NIL
go
T
1
T
OBSERVER
T

BUTTON
82
104
163
137
step
go
NIL
1
T
OBSERVER
T

@#$#@#$#@
WHAT IS IT?
-----------
This demonstrates how to have turtles move on a hexagonal lattice instead of NetLogo's usual square lattice.

Note that this technique only works as long as the turtles always stay on the vertices of the lattice.


HOW IT WORKS
------------
We treat the square lattice of patches as if it were a hex lattice, as follows:

- A turtle's heading must always be a multiple of 60.

- Turtles on even patch columns are offset down by half a patch.  (Since the south boundary of a patch is part of the patch, this does not move the turtle to a different patch.)

The resulting lattice has the correct structure, but distances and angles are distorted.  So the hexagons aren't actually regular, but it doesn't matter as long as the turtles move not using the built in FD command but using the HEX-FD procedure, and as long as you don't try to use primitives such as DISTANCE and TOWARDS.


CREDITS AND REFERENCES
----------------------
This example is for moving turtles.  "Hex Cells Example" shows how to make stationary hexagonal cells, such as for a cellular automaton.  (It would also be possible to combine both techniques in a single model.)
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Line -7566196 true 150 150 150 0
Line -7566196 true 121 23 180 23

hex
false
0
Polygon -7566196 true true 1 152 76 31 226 30 299 151 225 271 76 271

@#$#@#$#@
NetLogo 2.0.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
