turtles-own [ energy ]
patches-own [ grass ]
breeds [ cooperative-cows greedy-cows]

to setup
  ca
  setup-cows
  ask patches [
    set grass max-grass-height
    color-grass
  ]
  do-plotting
end

to setup-cows
  set-default-shape turtles "cow"   ;; applies to both breeds
  cct initial-cows [
    setxy (random-float screen-size-x)
          (random-float screen-size-y)
    set energy metabolism * 4
    ifelse (random-float 1.0 < cooperative-probability) [
      set breed cooperative-cows
      set color pink
    ] [
      set breed greedy-cows
      set color blue
    ]
  ]
end

to go
  ask turtles [
    move
    eat
    reproduce
  ]
  ask patches [
    grow-grass
    color-grass
  ]
  do-plotting
end

to reproduce  ;; turtle proceduere
  if (energy > reproduction-threshold) [
    set energy energy - reproduction-cost
    hatch 1 [ ]
  ]
end

to grow-grass  ;; patch procedure
  ifelse ( grass >= low-high-threshold) [
    if (high-growth-chance >= (random-float 100)) [
      set grass grass + 1
    ]
  ][
    if (low-growth-chance >= (random-float 100)) [
      set grass grass + 1
    ]
  ]
  if (grass > max-grass-height) [
    set grass max-grass-height
  ]
end

to color-grass  ;; patch procedure
  set pcolor hsb 0.25 1.0 ((grass / max-grass-height) * 0.75 + 0.25)
end

to move  ;; turtle proceduere
  rt random-float 360
  fd stride-length
  set energy energy - metabolism
  if (energy < 0) [ die ]
end

to eat  ;; turtle procedure
  ifelse breed = cooperative-cows [
    eat-cooperative
  ] [
    if breed = greedy-cows [
      eat-greedy
    ]
  ]
end

to eat-cooperative  ;; turtle proceduere
  if (grass > low-high-threshold) [
    set grass grass - 1
    set energy energy + grass-energy
  ]
end

to eat-greedy  ;; turtle proceduere
  if (grass > 0) [
    set grass grass - 1
    set energy energy + grass-energy
  ]
end

to do-plotting
  set-current-plot "Cows over time"
  set-current-plot-pen "cooperative"
  plot count cooperative-cows
  set-current-plot-pen "greedy"
  plot count greedy-cows
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2003.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Cooperation model.
; http://ccl.northwestern.edu/netlogo/models/Cooperation.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/Cooperation
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
336
10
661
356
10
10
15.0
1
10
1
1
1

CC-WINDOW
336
359
651
497
Command Center

BUTTON
218
42
273
75
go
go
T
1
T
OBSERVER

BUTTON
38
41
93
74
setup
setup
NIL
1
T
OBSERVER

SLIDER
7
115
214
148
cooperative-probability
cooperative-probability
0
1.0
0.5
0.01
1
NIL

SLIDER
7
80
154
113
initial-cows
initial-cows
0
100
20
1
1
NIL

SLIDER
174
464
331
497
low-high-threshold
low-high-threshold
0.0
99.0
5.0
1.0
1
NIL

SLIDER
7
431
175
464
high-growth-chance
high-growth-chance
0.0
99.0
77.0
1.0
1
NIL

SLIDER
155
80
332
113
stride-length
stride-length
0.0
0.3
0.08
0.01
1
NIL

SLIDER
174
431
331
464
max-grass-height
max-grass-height
1
40
10
1
1
NIL

SLIDER
7
182
215
215
reproduction-threshold
reproduction-threshold
0.0
200.0
102.0
1.0
1
NIL

SLIDER
173
398
331
431
grass-energy
grass-energy
0.0
200.0
51.0
1.0
1
NIL

SLIDER
216
115
332
148
metabolism
metabolism
0.0
99.0
6.0
1.0
1
NIL

SLIDER
7
464
175
497
low-growth-chance
low-growth-chance
0.0
99.0
30.0
1.0
1
NIL

SLIDER
7
150
215
183
reproduction-cost
reproduction-cost
0.0
99.0
54.0
1.0
1
NIL

PLOT
7
217
320
397
Cows over time
Time
Cows
0.0
10.0
0.0
100.0
true
true
PENS
"cooperative" 1.0 0 -65413 true
"greedy" 1.0 0 -16776961 true

TEXTBOX
113
62
203
80
basic sliders:

TEXTBOX
39
411
153
431
advanced sliders:

@#$#@#$#@
WHAT IS IT?
-----------
This model (and Altruism and Divide the Cake) are part of the EACH curriculum: "Evolution of Altruistic and Cooperative Habits: Learning About Complexity in Evolution".  See http://ccl.northwestern.edu/cm/EACH/ for more information.

This is an evolutionary biology model.  In it, agents (cows) compete for natural resources (grass).  Cows that are more successful in getting grass reproduce more often, and will thus be more evolutionarily successful.  This model includes two kinds of cows, greedy and cooperative.  It shows how these two different strategies do when competing against each other within a population that evolves over time.


HOW IT WORKS
------------
Every turn, each cow looks at the patch that it is currently on, and eats a unit of grass.  The greedy cows eat the grass regardless of the length of the grass on the current patch.  The cooperative cows won't eat the grass below a certain height.  This behavior is significant because below a certain height (called the "growth threshold"), the grass grows at a far slower rate than above it.  Thus, the cooperative agents leave more food for the overall population at a cost to their individual well-being, while the greedy agents eat the grass down to the nub, regardless of the effect on the overall population.


HOW TO USE IT?
--------------
GO: Starts and stops the model.

SETUP: Resets the simulation according to the parameters set by the sliders.

INITIAL-COWS: Sets the number of initial cows.

COOPERATIVE-PROBABILITY: Sets the chance an initial cow will be of the cooperative breed

STRIDE-LENGTH: This value determines the movement of the cows. Each cow will move forward STRIDE-LENGTH patches each turn.  As the value is increased, the cows will move to other patches more frequently.

GRASS-ENERGY: Each time a cow can eat some grass from the patch that it currently occupies, it increases its energy by the value of this slider.

METABOLISM: Every time step, each cow loses the amount of energy set by this slider.  If the cows energy reaches 0, it dies.  Every cow starts with a default energy of 50, which means it can go 50 / METABOLISM turns without eating.

REPRODUCTION-THRESHOLD: If a cow's energy reaches the value of this slider, it reproduces.  This value represents the food-gathering success that a cow would have to have in order to be able to reproduce.

REPRODUCTION-COST: Each time a cow reproduces, it loses the amount of energy set by this slider.  This value represents the energy cost of reproduction.

LOW-GROWTH-CHANCE: This value is the percentage chance that the grass below the growth threshold will grow back.  The higher this value, the less the discrepancy between the behaviors of the cooperative and greedy cows.

HIGH-GROWTH-CHANCE: This value is the percentage chance that the grass above the growth threshold will grow back.  The lower this value, the less the discrepancy between the behaviors of the cooperative and greedy cows.

MAX-GRASS-HEIGHT:  This value sets the highest length to which the grass can grow.

LOW-HIGH-THRESHOLD:  This value sets the grass growth threshold.  At, or above this value, the grass grows back with HIGH-GROWTH-CHANCE.  Below this value, the grass grows back with LOW-GROWTH-CHANCE.


THINGS TO NOTICE
----------------
Run the model with the default settings.  Watch the different growth curves on the population plot.  Which population expands first?  Which population wins in the end?


THINGS TO TRY
-------------
Slowly decrease the STRIDE-LENGTH slider. What happens to the populations?

At what value of STRIDE-LENGTH do the populations' growth rates change dramatically?  What does this indicate about the evolutionary advantages of Cooperating versus being Greedy?  What are the important environmental factors?

Change the METABOLISM and the GRASS-ENGERY values.  How do these values affect the model?

Change the LOW-GROWTH-CHANCE and the HIGH-GROWTH-CHANCE values.  How do these values affect the model?

How does the LOW-HIGH-THRESHOLD value affect the growth of the populations?

Can you find settings that maximize the advantage of the cooperative cows?


EXTENDING THE MODEL
-------------------
This model explores only one type of cooperative behavior, namely eating the grass above the growth threshold (the LOW-HIGH-THRESHOLD value).  What other cooperative, or altruistic, behaviors could be modeled that hurt individual fitness, while helping the group overall?  What other environmental conditions than grass length could be used to effect the health of a population?

This model relies primarily upon population "viscosity" (the STRIDE-LENGTH slider) to alter the behavior of the cows to allow for the success of the cooperative agents.  What other variables could have such a drastic effect on the evolutionary success of populations?

Also, consider that in this model the behaviors are fixed.  What would happen if the agents learned, or changed their behavior based on food availability?



NETLOGO FEATURES
----------------
Note the use of the HSB primitive in COLOR-GRASS.  This provides shading to different heights of grass.

Note how cows are created in the SETUP-COWS procedure. This is done to prevent NetLogo from giving one breed an advantage (as a result of the way NetLogo simulates parallel computations on a serial processor).  When a turtle is created, it is assigned an id number (stored in the turtle variable WHO) that is used by NetLogo to reference it.  When ASK is used, NetLogo gives each turtle a turn in order of their id numbers, from smallest to largest.  So if we had written SETUP-COWS like this:

|    to setup-cows
|      ...
|      create-custom-greedy-cows num-greedy [ ... ]
|      create-custom-cooperative-cows num-cooperative [ ... ]
|    end

then the initial greedy cows will all have id numbers that are lower than those of the initial cooperative cows, since id numbers are assigned in order.  This means the greedy cows will eat and reproduce first.  Theoretically this gives them an advantage.  As it happens, in most runs, the random movement of the cows would prevent this from having a significant effect on the model behavior.  Nonetheless, in the real code we intermingle the creation of the two types of cows in order to avoid any bias.

Future versions of NetLogo may support other scheduling methods besides "in order by id number".


RELATED MODELS
--------------
Altruism


CREDITS AND REFERENCES
-----------------------
This model and the Altruism model are part of the curriculum "Evolution of Altruistic and Cooperative Habits: Learning About Complexity in Evolution".  See http://ccl.northwestern.edu/cm/EACH/ for more information.

Thanks to Damon Centola, Eamon McKenzie, Josh Mitteldorf, and Scott Styles.

To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Cooperation model. http://ccl.northwestern.edu/netlogo/models/Cooperation. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 1998 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/Cooperation for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

cow
false
0
Polygon -7566196 true true 57 141 134 122 209 122 245 140 228 170 168 190 106 186 62 179 49 166
Polygon -7566196 true true 71 178 53 266 68 271 82 176
Polygon -7566196 true true 76 194 81 267 96 265 85 178
Polygon -7566196 true true 196 177 178 262 193 262 208 170
Polygon -7566196 true true 204 183 212 257 227 254 213 166
Polygon -7566196 true true 231 134 243 111 214 100 202 123
Polygon -7566196 true true 213 100 274 123 285 97 260 84 252 65 232 56 212 56 205 68 207 92 225 101
Polygon -7566196 true true 56 139 30 188 51 161

@#$#@#$#@
NetLogo 2.0alpha1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
