breeds      [ pacman ghosts pellets bonuses ]
pacman-own  [ new-heading ]
turtles-own [ home-pos ]
ghosts-own  [ eaten? ]
pellets-own [ powerup? ]
bonuses-own [ bonus countdown ]

patches-own [ pellet-grid? ]  ;; true/false: is a pellet here initially?

globals [
  level         ;; current level
  score         ;; your score
  lives         ;; remaining lives
  extra-lives   ;; total number of extra lives you've won
  scared        ;; time until ghosts aren't scared (0 means not scared)
  level-over?   ;; true when a level is complete
  dead?         ;; true when Pac-Man is loses a life
  next-bonus-in ;; time until next bonus is created
  tool which-ghost ;; variables needed to properly load levels 4 and above.
]

;;;;;;;;;;;;;;;;;;;;;;;;
;;; Setup Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

to new  ;; Observer Button
  ca
  set level 1
  load-map
  set score 0
  set lives 3
  set extra-lives 0
  set scared 0
  set level-over? false
end

to load-map  ;; Observer Procedure
  locals [maps current-score current-lives current-extra-lives current-difficulty]
  ;; Filenames of Level Files
  set maps ["pacmap1.csv" "pacmap2.csv" "pacmap3.csv"
            "pacmap4.csv" "pacmap5.csv"]
  set current-score score
  set current-lives lives
  set current-extra-lives extra-lives
  set current-difficulty difficulty

  ifelse ((level - 1) < length maps)
  [ import-world item (level - 1) maps
    set score current-score
    set lives current-lives
    set extra-lives current-extra-lives
    set difficulty current-difficulty
    set dead? false
    ask pacman
    [ set home-pos list xcor ycor ]
    ask ghosts
    [ set home-pos list xcor ycor ]
  ]
  [ set level 1
    load-map ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Runtime Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

to play  ;; Observer Forever Button
  ;; Only true at this point if you died and are trying to continue
  if dead?
  [ stop ]
  every (1 - difficulty / 10)
  [ move-pacman ]
  every 0.25
  [ update-bonuses ]
  if floor (score / 35000) > extra-lives
  [ set lives lives + 1
    set extra-lives extra-lives + 1 ]
  if dead?
  [ ifelse lives = 0
    [ user-message "Game Over!\nScore: " + score ]
    [ set lives lives - 1
      ifelse lives = 0
      [ user-message "You died!\nNo lives left." ]
      [ ifelse lives = 1
        [ user-message "You died!\nOnly 1 life left." ]
        [ user-message "You died!\nOnly " + lives + " lives left." ]
      ]
      ask pacman
      [ setxy (item 0 home-pos) (item 1 home-pos)
        set heading 0
      ]
      ask ghosts
      [ setxy (item 0 home-pos) (item 1 home-pos)
        set heading 0
        set shape "ghost"
      ]
      set dead? false
    ]
    stop
  ]
  if level-over?
  [ user-message "Level Complete!\nScore: " + score  ;; \n means start a new line
    no-display
    set level level + 1
    load-map
    display
    set level-over? false
    stop ]
  every 1.6 * (1 - difficulty / 10)
  [ move-ghosts ]
  every next-bonus-in
  [ create-bonus ]
end

to move-pacman  ;; Observer Procedure
  locals [old-heading]
  no-display
  ask pacman
  [ ;; move forward unless blocked by wall
    set old-heading heading
    set heading new-heading
    if pcolor-of patch-ahead 1 != black
    [ set heading old-heading ]
    if pcolor-of patch-ahead 1 = black
    [ fd 1 ]
    consume
    ;; Level ends when all pellets are eaten
    if not any? pellets
    [ set level-over? true ]
    ;; Animation
    ifelse shape = "pacman"
    [ set shape "pacman-open" ]
    [ set shape "pacman" ]
  ]
  display
end

to consume  ;; Pacman Procedure
  ;; Consume Bonuses
  if any? bonuses-here
  [ set score score + sum values-from bonuses-here [bonus]
    ask bonuses-here [ die ] ]

  ;; Consume Pellets
  if any? pellets-here
  [ ifelse powerup?-of one-of pellets-here
    [ set score score + 500
      set scared 40
      ask ghosts
      [ if not eaten?
        [ set shape "scared" ] ]
    ]
    [ set score score + 100 ]
    ask pellets-here [ die ] ]

  ;; Ghosts
  if any? ghosts-here with [not eaten?]
  [ ifelse scared = 0
    [ set dead? true ]
    [ ask ghosts-here with [not eaten?]
      [ set eaten? true
        set shape "eyes"
        set score score + 500 ]
    ]
  ]
end

to update-bonuses  ;; Observer Procedure
  ask bonuses
  [ set heading heading + 13
    set countdown countdown - 1
    if countdown = 0
    [ die ] ]
end

to move-ghosts  ;; Observer Procedure
  ask ghosts
  [ ifelse eaten?
    [ if pcolor-of patch-at 0 1 = gray
      [ set eaten? false
        set shape "ghost" ]
      return-home
    ]
    [ choose-heading ]
    fd 1
  ]
  if scared > 0
  [ set scared scared - 1
    ifelse scared < 10 and scared mod 2 = 0
    [ ask ghosts with [not eaten?]
      [ set shape "ghost" ] ]
    [ ask ghosts with [not eaten?]
      [ set shape "scared" ] ]
    if scared = 0
    [ ask ghosts with [not eaten?]
      [ set shape "ghost" ]
    ]
  ]
end

to return-home  ;; Ghosts Procedure
  locals [dirs new-dirs home-dir home-path]
  set dirs clear-headings
  set new-dirs remove opposite heading dirs
  ifelse pcolor = gray
    [ set home-dir 0 ]
    [ set home-dir towards-nowrap one-of patches with [pcolor = gray] ]
  set home-path 90 * round (home-dir / 90)

  if length new-dirs = 1
  [ set heading item 0 new-dirs ]
  if length new-dirs > 1
  [ ifelse position home-path new-dirs != false
    [ set heading home-path ]
    [ set heading random-one-of new-dirs ]
  ]
end

to choose-heading  ;; Ghosts Procedure
  locals [dirs new-dirs pacman-dir]
  set dirs clear-headings
  set new-dirs remove opposite heading dirs
  set pacman-dir false

  if length dirs = 1
  [ set heading item 0 dirs ]
  if length dirs = 2
  [ ifelse see-pacman item 0 dirs
    [ set pacman-dir item 0 dirs ]
    [ ifelse see-pacman item 1 dirs
      [ set pacman-dir item 1 dirs ]
      [ set heading random-one-of new-dirs ]
    ]
  ]
  if length dirs = 3
  [ ifelse see-pacman item 0 dirs
    [ set pacman-dir item 0 dirs ]
    [ ifelse see-pacman item 1 dirs
      [ set pacman-dir item 1 dirs ]
      [ ifelse see-pacman item 2 dirs
        [ set pacman-dir item 2 dirs ]
        [ set heading random-one-of new-dirs ]
      ]
    ]
  ]
  if length dirs = 4
  [ ifelse see-pacman item 0 dirs
    [ set pacman-dir item 0 dirs ]
    [ ifelse see-pacman item 1 dirs
      [ set pacman-dir item 1 dirs ]
      [ ifelse see-pacman item 2 dirs
        [ set pacman-dir item 2 dirs ]
        [ ifelse see-pacman item 3 dirs
          [ set pacman-dir item 3 dirs ]
          [ set heading random-one-of new-dirs ]
        ]
      ]
    ]
  ]
  if pacman-dir != false
  [ ifelse scared = 0
    [ set heading pacman-dir ]
    [ set dirs remove pacman-dir dirs
      set heading random-one-of dirs
    ]
  ]
end

to-report clear-headings ;; ghosts procedure
  locals [dirs]
  set dirs []
  if pcolor-of patch-at 0 1 != blue
  [ set dirs lput 0 dirs ]
  if pcolor-of patch-at 1 0 != blue
  [ set dirs lput 90 dirs ]
  if pcolor-of patch-at 0 -1 != blue
  [ set dirs lput 180 dirs ]
  if pcolor-of patch-at -1 0 != blue
  [ set dirs lput 270 dirs ]
  report dirs
end

to-report opposite [dir]
  ifelse dir < 180
  [ report dir + 180 ]
  [ report dir - 180 ]
end

to-report see-pacman [dir] ;; ghosts procedure
  locals [p saw-pacman?]
  set saw-pacman? false
  set p patch-here
  while [pcolor-of p = black]
  [ ask p
    [ if any? pacman-here
      [ set saw-pacman? true ]
      set p patch-at sin dir cos dir ;; next patch in direction dir
    ]
    ;; stop looking if you loop around the whole screen
    if p = patch-here [ report saw-pacman? ]
  ]
  report saw-pacman?
end
  
to create-bonus ;; Observer Procedure
  locals [bonus-patch]
  ifelse next-bonus-in = 0
  [ set next-bonus-in 10 ]
  [ set bonus-patch random-one-of patches with [pellet-grid? and
                                                not any? bonuses-here and
                                                not any? pellets-here]
    if bonus-patch != nobody
    [ ask bonus-patch
      [ sprout 1
        [ set breed bonuses
          set shape "star"
          set heading 0
          set color random 14 * 10 + 5
          set bonus (random 10 + 1) * 100
          set countdown random 200 + 50 ] ]
      set next-bonus-in 5 + random 10 ] ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Interface Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to move-up
  ask pacman [ set new-heading 0 ]
end

to move-right
  ask pacman [ set new-heading 90 ]
end

to move-down
  ask pacman [ set new-heading 180 ]
end

to move-left
  ask pacman [ set new-heading 270 ]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2001 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2001).  NetLogo Pac-Man model.
; http://ccl.northwestern.edu/netlogo/models/Pac-Man.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/Pac-Man
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
243
10
694
482
10
10
21.0
1
10
1
1
1

CC-WINDOW
11
359
231
461
Command Center

MONITOR
11
41
121
90
Score
score
0
1

BUTTON
11
90
121
123
New
new
NIL
1
T
OBSERVER
T

BUTTON
121
90
231
123
Play
play
T
1
T
OBSERVER
T

BUTTON
94
165
149
198
Up
move-up
NIL
1
T
OBSERVER
T

BUTTON
120
198
175
231
Right
move-right
NIL
1
T
OBSERVER
T

BUTTON
95
231
150
264
Down
move-down
NIL
1
T
OBSERVER
T

BUTTON
66
198
121
231
Left
move-left
NIL
1
T
OBSERVER
T

MONITOR
121
41
176
90
Level
level
0
1

MONITOR
176
41
231
90
Lives
lives
0
1

SLIDER
11
123
231
156
difficulty
difficulty
0
7
4
1
1
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This is the classic arcade game, Pac-Man.  The game involves navigating Pac-Man through a maze.  Your objective is that Pac-Man eat all of the pellets (white circles), while avoiding the ghosts that pursue him.

If a ghost ever catches Pac-Man then Pac-Man is defeated.  If this occurs, the level will reset, but this will happen only if Pac-Man still has some lives remaining. (The pellets already collected on the level remain collected.)

However, when Pac-Man eats a Power-Pellet (large white circle) he can turn the tide, and the ghosts will turn scared and flee from him, for with the power of the Power-Pellet, Pac-Man can eat the ghosts!  Once a ghost is eaten it will return to its base, where it is born again, immune to the Power-Pellet until Pac-Man can find a new one to consume.  Pac-Man had better do just that, because unfortunately, the power of the Power-Pellet does not last forever, and will begin to wear off over time. (You will see the ghosts start to flash back to their normal appearance during the last few seconds of the Power-Pellet's effectiveness.)

Finally, occasionally a bonus (rotating star) will appear in the maze.  This bonus gives Pac-Man extra points if he eats it, but it will disappear if Pac-Man doesn't get it within a limited amount of time.


HOW TO USE IT
-------------
Monitors
-- SCORE shows your current score.  You get points for collecting pellets, eating ghosts, and collecting bonuses.  You will get an extra life after every 35,000 points.
-- LEVEL shows your current level.  Each level has a different map, if you complete all the maps, it will loop back to the first map and continue.
-- LIVES shows how many extra lives you have remaining.  If you are defeated by a ghost when this is at 0, the game is over.

Sliders
-- DIFFICULTY controls the speed of the game.  Lower numbers make both the ghosts and Pac-Man move slowly, giving you more time to react as you play.

Buttons
-- NEW sets up a new game on level 1, with 3 lives, and a score of 0.
-- PLAY begins the game.  The game will pause after each level, so you will need to hit PLAY again after each level to continue.

Controls
-- UP, DOWN, LEFT, RIGHT control the direction Pac-Man moves.


THINGS TO NOTICE
----------------
If you go off the edge of the maze you will wrap around to the other side.

Identifing Things in the Maze:
-- Yellow Circle with a mouth:  This is Pac-Man - you.
-- White Circles:               These are Pellets - Collect all of these (including the Power-Pellets) to move on to the next level.
-- Large White Circles:         These are Power-Pellets - They allow you to eat the Ghosts for a limited ammount of time.
-- Blue Squares:                These are the walls of the maze - Neither Pac-Man nor the Ghosts can move through the walls.
-- Gray Squares:                These are the Ghost Gates - Only Ghosts can move through them, and if they do so after having been eaten they will be healed.
-- Rotating Colored Stars:      These are Bonus Stars - They give you extra points when you eat them.
-- Colorful Ghost with Eyes:    These are the active Ghosts - Watch out for them!
-- Blue Ghost Shape:            These are the scared Ghosts - Eat them for Extra Points!
-- Two Small Eyes:              These are the Ghosts after they've been eaten - They will not affect you, and you can't eat them again, so just ignore them, but try not to be near its base when it gets back there.

Scoring System
-- Eat a Pellet:       100 Points
-- Eat a Power-Pellet: 500 Points
-- Eat a Scared Ghost: 500 Points
-- Eat a Bonus Star:   100-1000 Points (varies)


THINGS TO TRY
-------------
Beat your Highest Score.

Can you write an automated program for Pac-Man that will get him safely through the maze and collet all the pellets?


EXTENDING THE MODEL
-------------------
Think of other power-ups or bonuses that might be fun to have and make them appear randomly in the maze.

Add new enemies that behave differently from the ghosts. 


NETLOGO FEATURES
-----------------
This model makes use of breeds, create-custom-<breed>, every, and user-message.

The "import-world" command is used to read in the different maze configurations (levels).


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2002).  NetLogo Pac-Man model. http://ccl.northwestern.edu/netlogo/models/Pac-Man. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2002 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/Pac-Man for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7566196 true true 35 35 230

eyes
false
0
Circle -1 true false 62 75 57
Circle -1 true false 182 75 57
Circle -16777216 true false 79 93 20
Circle -16777216 true false 196 93 21

ghost
false
0
Circle -7566196 true true 61 30 179
Rectangle -7566196 true true 60 120 240 232
Polygon -7566196 true true 60 229 60 284 105 239 149 284 195 240 239 285 239 228 60 229 60 229 60 229
Circle -1 true false 81 78 56
Circle -16777216 true false 99 98 19
Circle -1 true false 155 80 56
Circle -16777216 true false 171 98 17

pacman
true
0
Circle -7566196 true true 35 35 230
Polygon -16777216 true false 120 30 148 149 180 30

pacman-open
true
0
Circle -7566196 true true 35 35 230
Polygon -16777216 true false 225 30 149 152 75 30

pellet
true
0
Circle -7566196 true true 121 121 60

scared
false
0
Circle -16776961 true false 61 30 179
Rectangle -16776961 true false 60 120 240 232
Polygon -16776961 true false 60 229 60 284 105 239 149 284 195 240 239 285 239 228 60 229 60 229 60 229
Circle -16777216 true false 81 78 56
Circle -16777216 true false 155 80 56
Line -16777216 false 137 193 102 166
Line -16777216 false 103 166 75 194
Line -16777216 false 138 193 171 165
Line -16777216 false 172 166 198 192

star
true
0
Polygon -7566196 true true 74 254 149 16 224 254 29 104 270 104

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
new
@#$#@#$#@
@#$#@#$#@
